from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.TCollection import *
from OCC.Core.TDocStd import *
from OCC.Core.Message import *
from OCC.Core.TDF import *
from OCC.Core.gp import *

#the following typedef cannot be wrapped as is
TObj_TIntSparseArray_MapOfData = NewType('TObj_TIntSparseArray_MapOfData', Any)
#the following typedef cannot be wrapped as is
TObj_TIntSparseArray_VecOfData = NewType('TObj_TIntSparseArray_VecOfData', Any)

class TObj_SequenceOfIterator:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class TObj_SequenceOfObject:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> False: ...
    def Last(self) -> False: ...
    def Length(self) -> int: ...
    def Append(self, theItem: False) -> False: ...
    def Prepend(self, theItem: False) -> False: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> False: ...
    def SetValue(self, theIndex: int, theValue: False) -> None: ...

class TObj_DeletingMode(IntEnum):
	TObj_FreeOnly: int = ...
	TObj_KeepDepending: int = ...
	TObj_Forced: int = ...
TObj_FreeOnly = TObj_DeletingMode.TObj_FreeOnly
TObj_KeepDepending = TObj_DeletingMode.TObj_KeepDepending
TObj_Forced = TObj_DeletingMode.TObj_Forced

class TObj_Application(TDocStd_Application):
	def CreateNewDocument(self, theDoc: TDocStd_Document, theFormat: TCollection_ExtendedString) -> bool: ...
	@overload
	def ErrorMessage(self, theMsg: TCollection_ExtendedString, theLevel: Message_Gravity) -> None: ...
	@overload
	def ErrorMessage(self, theMsg: TCollection_ExtendedString) -> None: ...
	@staticmethod
	def GetInstance() -> TObj_Application: ...
	def IsVerbose(self) -> bool: ...
	def LoadDocument(self, theSourceFile: TCollection_ExtendedString, theTargetDoc: TDocStd_Document) -> bool: ...
	def Messenger(self) -> Message_Messenger: ...
	def ResourcesName(self) -> str: ...
	def SaveDocument(self, theSourceDoc: TDocStd_Document, theTargetFile: TCollection_ExtendedString) -> bool: ...
	def SetVerbose(self, isVerbose: bool) -> None: ...

class TObj_Assistant:
	@staticmethod
	def BindModel(theModel: TObj_Model) -> None: ...
	@staticmethod
	def BindType(theType: Standard_Type) -> int: ...
	@staticmethod
	def ClearModelMap() -> None: ...
	@staticmethod
	def ClearTypeMap() -> None: ...
	@staticmethod
	def FindModel(theName: str) -> TObj_Model: ...
	@staticmethod
	def FindType(theTypeIndex: int) -> Standard_Type: ...
	@staticmethod
	def FindTypeIndex(theType: Standard_Type) -> int: ...
	@staticmethod
	def GetAppVersion() -> int: ...
	@staticmethod
	def GetCurrentModel() -> TObj_Model: ...
	@staticmethod
	def SetCurrentModel(theModel: TObj_Model) -> None: ...
	@staticmethod
	def UnSetCurrentModel() -> None: ...

class TObj_CheckModel(Message_Algorithm):
	def __init__(self, theModel: TObj_Model) -> None: ...
	def GetModel(self) -> TObj_Model: ...
	def IsToFix(self) -> bool: ...
	def Perform(self) -> bool: ...
	def SetToFix(self, theToFix: bool) -> None: ...

class TObj_Model(Standard_Transient):
	def AbortCommand(self) -> None: ...
	def Close(self) -> bool: ...
	def CloseDocument(self, theDoc: TDocStd_Document) -> None: ...
	def CommitCommand(self) -> None: ...
	def CopyReferences(self, theTarget: TObj_Model, theRelocTable: TDF_RelocationTable) -> None: ...
	def FindObject(self, theName: TCollection_HExtendedString, theDictionary: TObj_TNameContainer) -> TObj_Object: ...
	def GetApplication(self) -> TObj_Application: ...
	def GetChecker(self) -> TObj_CheckModel: ...
	def GetChildren(self) -> TObj_ObjectIterator: ...
	def GetDictionary(self) -> TObj_TNameContainer: ...
	def GetDocument(self) -> TDocStd_Document: ...
	@staticmethod
	def GetDocumentModel(theLabel: TDF_Label) -> TObj_Model: ...
	def GetFile(self) -> TCollection_HExtendedString: ...
	def GetFormat(self) -> TCollection_ExtendedString: ...
	def GetFormatVersion(self) -> int: ...
	def GetGUID(self) -> Standard_GUID: ...
	def GetLabel(self) -> TDF_Label: ...
	def GetMainPartition(self) -> TObj_Partition: ...
	def GetModelName(self) -> TCollection_HExtendedString: ...
	def GetObjects(self) -> TObj_ObjectIterator: ...
	def GetRoot(self) -> TObj_Object: ...
	def HasOpenCommand(self) -> bool: ...
	def IsModified(self) -> bool: ...
	def IsRegisteredName(self, theName: TCollection_HExtendedString, theDictionary: TObj_TNameContainer) -> bool: ...
	def Load(self, theFile: TCollection_ExtendedString) -> bool: ...
	def Messenger(self) -> Message_Messenger: ...
	def NewEmpty(self) -> TObj_Model: ...
	def OpenCommand(self) -> None: ...
	def Paste(self, theModel: TObj_Model, theRelocTable: Optional[TDF_RelocationTable] = 0) -> bool: ...
	def RegisterName(self, theName: TCollection_HExtendedString, theLabel: TDF_Label, theDictionary: TObj_TNameContainer) -> None: ...
	def Save(self) -> bool: ...
	def SaveAs(self, theFile: TCollection_ExtendedString) -> bool: ...
	def SetLabel(self, theLabel: TDF_Label) -> None: ...
	def SetMessenger(self, theMsgr: Message_Messenger) -> None: ...
	def SetModified(self, theModified: bool) -> None: ...
	@staticmethod
	def SetNewName(theObject: TObj_Object) -> None: ...
	def UnRegisterName(self, theName: TCollection_HExtendedString, theDictionary: TObj_TNameContainer) -> None: ...
	def Update(self) -> bool: ...

class TObj_Object(Standard_Transient):
	def AddBackReference(self, theObject: TObj_Object) -> None: ...
	def AfterRetrieval(self) -> None: ...
	def BeforeStoring(self) -> None: ...
	def CanDetach(self, theMode: Optional[TObj_DeletingMode] = TObj_FreeOnly) -> bool: ...
	def CanRemoveReference(self, theObject: TObj_Object) -> bool: ...
	def ClearBackReferences(self) -> None: ...
	def ClearFlags(self, theMask: Optional[int] = ~0) -> None: ...
	def Clone(self, theTargetLabel: TDF_Label, theRelocTable: Optional[TDF_RelocationTable] = 0) -> TObj_Object: ...
	def CopyChildren(self, theTargetLabel: TDF_Label, theRelocTable: TDF_RelocationTable) -> None: ...
	def CopyReferences(self, theTargetObject: TObj_Object, theRelocTable: TDF_RelocationTable) -> None: ...
	@overload
	def Detach(self, theMode: Optional[TObj_DeletingMode] = TObj_FreeOnly) -> bool: ...
	@overload
	@staticmethod
	def Detach(theLabel: TDF_Label, theMode: Optional[TObj_DeletingMode] = TObj_FreeOnly) -> bool: ...
	def GetBackReferences(self, theType: Optional[Standard_Type] = None) -> TObj_ObjectIterator: ...
	def GetBadReference(self, theRoot: TDF_Label, theBadReference: TDF_Label) -> bool: ...
	def GetChildLabel(self) -> TDF_Label: ...
	def GetChildren(self, theType: Optional[Standard_Type] = None) -> TObj_ObjectIterator: ...
	def GetDataLabel(self) -> TDF_Label: ...
	def GetDictionary(self) -> TObj_TNameContainer: ...
	def GetFatherObject(self, theType: Optional[Standard_Type] = None) -> TObj_Object: ...
	def GetFlags(self) -> int: ...
	def GetLabel(self) -> TDF_Label: ...
	def GetModel(self) -> TObj_Model: ...
	@overload
	def GetName(self) -> TCollection_HExtendedString: ...
	@overload
	def GetName(self, theName: TCollection_ExtendedString) -> bool: ...
	@overload
	def GetName(self, theName: TCollection_AsciiString) -> bool: ...
	@staticmethod
	def GetObj(theLabel: TDF_Label, theResult: TObj_Object, isSuper: Optional[bool] = False) -> bool: ...
	def GetOrder(self) -> int: ...
	def GetReferenceLabel(self) -> TDF_Label: ...
	def GetReferences(self, theType: Optional[Standard_Type] = None) -> TObj_ObjectIterator: ...
	def GetTypeFlags(self) -> int: ...
	def HasBackReferences(self) -> bool: ...
	def HasModifications(self) -> bool: ...
	def HasReference(self, theObject: TObj_Object) -> bool: ...
	def IsAlive(self) -> bool: ...
	def RelocateReferences(self, theFromRoot: TDF_Label, theToRoot: TDF_Label, theUpdateBackRefs: Optional[bool] = True) -> bool: ...
	def RemoveAllReferences(self) -> None: ...
	def RemoveBackReference(self, theObject: TObj_Object, theSingleOnly: Optional[bool] = True) -> None: ...
	def RemoveBackReferences(self, theMode: Optional[TObj_DeletingMode] = TObj_FreeOnly) -> bool: ...
	def RemoveReference(self, theObject: TObj_Object) -> None: ...
	def ReplaceReference(self, theOldObject: TObj_Object, theNewObject: TObj_Object) -> None: ...
	def SetFlags(self, theMask: int) -> None: ...
	@overload
	def SetName(self, theName: TCollection_HExtendedString) -> bool: ...
	@overload
	def SetName(self, theName: TCollection_HAsciiString) -> bool: ...
	@overload
	def SetName(self, name: str) -> bool: ...
	def SetOrder(self, theIndx: int) -> bool: ...
	def TestFlags(self, theMask: int) -> bool: ...
	def getChildLabel(self, theRank: int) -> TDF_Label: ...

class TObj_ObjectIterator(Standard_Transient):
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def Value(self) -> TObj_Object: ...

class TObj_Persistence:
	@staticmethod
	def CreateNewObject(theType: str, theLabel: TDF_Label) -> TObj_Object: ...

class TObj_TIntSparseArray(TDF_Attribute):
	def __init__(self) -> None: ...
	def AfterUndo(self, theDelta: TDF_AttributeDelta, toForce: bool) -> bool: ...
	def BackupCopy(self) -> TDF_Attribute: ...
	def BeforeCommitTransaction(self) -> None: ...
	def Clear(self) -> None: ...
	def ClearDelta(self) -> None: ...
	def DeltaOnModification(self, theDelta: TDF_DeltaOnModification) -> None: ...
	@staticmethod
	def GetID() -> Standard_GUID: ...
	def GetIterator(self) -> False: ...
	def HasValue(self, theId: int) -> bool: ...
	def ID(self) -> Standard_GUID: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def Paste(self, theInto: TDF_Attribute, theRT: TDF_RelocationTable) -> None: ...
	def Restore(self, theDelta: TDF_Attribute) -> None: ...
	@staticmethod
	def Set(theLabel: TDF_Label) -> TObj_TIntSparseArray: ...
	def SetDoBackup(self, toDo: bool) -> None: ...
	def SetValue(self, theId: int, theValue: int) -> None: ...
	def Size(self) -> int: ...
	def UnsetValue(self, theId: int) -> None: ...
	def Value(self, theId: int) -> int: ...

class TObj_TModel(TDF_Attribute):
	def __init__(self) -> None: ...
	@staticmethod
	def GetID() -> Standard_GUID: ...
	def ID(self) -> Standard_GUID: ...
	def Model(self) -> TObj_Model: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def Paste(self, theInto: TDF_Attribute, theRT: TDF_RelocationTable) -> None: ...
	def Restore(self, theWith: TDF_Attribute) -> None: ...
	def Set(self, theModel: TObj_Model) -> None: ...

class TObj_TNameContainer(TDF_Attribute):
	def __init__(self) -> None: ...
	def Clear(self) -> None: ...
	def Get(self) -> TObj_DataMapOfNameLabel: ...
	@staticmethod
	def GetID() -> Standard_GUID: ...
	def ID(self) -> Standard_GUID: ...
	def IsRegistered(self, theName: TCollection_HExtendedString) -> bool: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def Paste(self, theInto: TDF_Attribute, theRT: TDF_RelocationTable) -> None: ...
	def RecordName(self, theName: TCollection_HExtendedString, theLabel: TDF_Label) -> None: ...
	def RemoveName(self, theName: TCollection_HExtendedString) -> None: ...
	def Restore(self, theWith: TDF_Attribute) -> None: ...
	@overload
	@staticmethod
	def Set(theLabel: TDF_Label) -> TObj_TNameContainer: ...
	@overload
	def Set(self, theElem: TObj_DataMapOfNameLabel) -> None: ...

class TObj_TObject(TDF_Attribute):
	def __init__(self) -> None: ...
	def AfterUndo(self, anAttDelta: TDF_AttributeDelta, forceIt: bool) -> bool: ...
	def BeforeForget(self) -> None: ...
	def Get(self) -> TObj_Object: ...
	@staticmethod
	def GetID() -> Standard_GUID: ...
	def ID(self) -> Standard_GUID: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def Paste(self, theInto: TDF_Attribute, theRT: TDF_RelocationTable) -> None: ...
	def Restore(self, theWith: TDF_Attribute) -> None: ...
	@overload
	@staticmethod
	def Set(theLabel: TDF_Label, theElem: TObj_Object) -> TObj_TObject: ...
	@overload
	def Set(self, theElem: TObj_Object) -> None: ...

class TObj_TReference(TDF_Attribute):
	def __init__(self) -> None: ...
	def AfterResume(self) -> None: ...
	def AfterRetrieval(self, forceIt: Optional[bool] = False) -> bool: ...
	def AfterUndo(self, theDelta: TDF_AttributeDelta, isForced: Optional[bool] = False) -> bool: ...
	def BeforeForget(self) -> None: ...
	def BeforeUndo(self, theDelta: TDF_AttributeDelta, isForced: Optional[bool] = False) -> bool: ...
	def Get(self) -> TObj_Object: ...
	@staticmethod
	def GetID() -> Standard_GUID: ...
	def GetLabel(self) -> TDF_Label: ...
	def GetMasterLabel(self) -> TDF_Label: ...
	def ID(self) -> Standard_GUID: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def Paste(self, theInto: TDF_Attribute, theRT: TDF_RelocationTable) -> None: ...
	def Restore(self, theWith: TDF_Attribute) -> None: ...
	@overload
	@staticmethod
	def Set(theLabel: TDF_Label, theObject: TObj_Object, theMaster: TObj_Object) -> TObj_TReference: ...
	@overload
	def Set(self, theObject: TObj_Object, theMasterLabel: TDF_Label) -> None: ...
	@overload
	def Set(self, theLabel: TDF_Label, theMasterLabel: TDF_Label) -> None: ...

class TObj_TXYZ(TDF_Attribute):
	def __init__(self) -> None: ...
	def Get(self) -> gp_XYZ: ...
	@staticmethod
	def GetID() -> Standard_GUID: ...
	def ID(self) -> Standard_GUID: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	def Paste(self, theInto: TDF_Attribute, theRT: TDF_RelocationTable) -> None: ...
	def Restore(self, theWith: TDF_Attribute) -> None: ...
	@overload
	@staticmethod
	def Set(theLabel: TDF_Label, theXYZ: gp_XYZ) -> TObj_TXYZ: ...
	@overload
	def Set(self, theXYZ: gp_XYZ) -> None: ...

class TObj_LabelIterator(TObj_ObjectIterator):
	def LabelValue(self) -> TDF_Label: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def Value(self) -> TObj_Object: ...

class TObj_ModelIterator(TObj_ObjectIterator):
	def __init__(self, theModel: TObj_Model) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def Value(self) -> TObj_Object: ...

class TObj_Partition(TObj_Object):
	def AfterRetrieval(self) -> None: ...
	@staticmethod
	def Create(theLabel: TDF_Label, theSetName: Optional[bool] = True) -> TObj_Partition: ...
	def GetLastIndex(self) -> int: ...
	def GetNamePrefix(self) -> TCollection_HExtendedString: ...
	def GetNewName(self, theIsToChangeCount: Optional[bool] = True) -> TCollection_HExtendedString: ...
	@staticmethod
	def GetPartition(theObject: TObj_Object) -> TObj_Partition: ...
	def NewLabel(self) -> TDF_Label: ...
	def SetLastIndex(self, theIndex: int) -> None: ...
	def SetName(self, theName: TCollection_HExtendedString) -> bool: ...
	def SetNamePrefix(self, thePrefix: TCollection_HExtendedString) -> None: ...
	def Update(self) -> bool: ...

class TObj_SequenceIterator(TObj_ObjectIterator):
	def __init__(self, theObjects: TObj_HSequenceOfObject, theType: Optional[Standard_Type] = None) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def Value(self) -> TObj_Object: ...

class TObj_HiddenPartition(TObj_Partition):
	def __init__(self, theLabel: TDF_Label) -> None: ...
	def GetTypeFlags(self) -> int: ...

class TObj_OcafObjectIterator(TObj_LabelIterator):
	def __init__(self, theLabel: TDF_Label, theType: Optional[Standard_Type] = None, theRecursive: Optional[bool] = False, theAllSubChildren: Optional[bool] = False) -> None: ...

class TObj_ReferenceIterator(TObj_LabelIterator):
	def __init__(self, theLabel: TDF_Label, theType: Optional[Standard_Type] = None, theRecursive: Optional[bool] = True) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes

class TObj_HSequenceOfObject(TObj_SequenceOfObject, Standard_Transient):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, other: TObj_SequenceOfObject) -> None: ...
    def Sequence(self) -> TObj_SequenceOfObject: ...
    def Append(self, theSequence: TObj_SequenceOfObject) -> None: ...


TObj_Application_GetInstance = TObj_Application.GetInstance
TObj_Assistant_BindModel = TObj_Assistant.BindModel
TObj_Assistant_BindType = TObj_Assistant.BindType
TObj_Assistant_ClearModelMap = TObj_Assistant.ClearModelMap
TObj_Assistant_ClearTypeMap = TObj_Assistant.ClearTypeMap
TObj_Assistant_FindModel = TObj_Assistant.FindModel
TObj_Assistant_FindType = TObj_Assistant.FindType
TObj_Assistant_FindTypeIndex = TObj_Assistant.FindTypeIndex
TObj_Assistant_GetAppVersion = TObj_Assistant.GetAppVersion
TObj_Assistant_GetCurrentModel = TObj_Assistant.GetCurrentModel
TObj_Assistant_SetCurrentModel = TObj_Assistant.SetCurrentModel
TObj_Assistant_UnSetCurrentModel = TObj_Assistant.UnSetCurrentModel
TObj_Model_GetDocumentModel = TObj_Model.GetDocumentModel
TObj_Model_SetNewName = TObj_Model.SetNewName
TObj_Object_Detach = TObj_Object.Detach
TObj_Object_GetObj = TObj_Object.GetObj
TObj_Persistence_CreateNewObject = TObj_Persistence.CreateNewObject
TObj_TIntSparseArray_GetID = TObj_TIntSparseArray.GetID
TObj_TIntSparseArray_Set = TObj_TIntSparseArray.Set
TObj_TModel_GetID = TObj_TModel.GetID
TObj_TNameContainer_GetID = TObj_TNameContainer.GetID
TObj_TNameContainer_Set = TObj_TNameContainer.Set
TObj_TObject_GetID = TObj_TObject.GetID
TObj_TObject_Set = TObj_TObject.Set
TObj_TReference_GetID = TObj_TReference.GetID
TObj_TReference_Set = TObj_TReference.Set
TObj_TXYZ_GetID = TObj_TXYZ.GetID
TObj_TXYZ_Set = TObj_TXYZ.Set
TObj_Partition_Create = TObj_Partition.Create
TObj_Partition_GetPartition = TObj_Partition.GetPartition



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.TCollection import *
from OCC.Core.XCAFPrs import *
from OCC.Core.TDocStd import *
from OCC.Core.TDF import *
from OCC.Core.TColStd import *
from OCC.Core.Message import *
from OCC.Core.Poly import *
from OCC.Core.Quantity import *
from OCC.Core.Bnd import *
from OCC.Core.XCAFDoc import *
from OCC.Core.Image import *


class RWGltf_GltfPrimitiveMode(IntEnum):
	RWGltf_GltfPrimitiveMode_UNKNOWN: int = ...
	RWGltf_GltfPrimitiveMode_Points: int = ...
	RWGltf_GltfPrimitiveMode_Lines: int = ...
	RWGltf_GltfPrimitiveMode_LineLoop: int = ...
	RWGltf_GltfPrimitiveMode_LineStrip: int = ...
	RWGltf_GltfPrimitiveMode_Triangles: int = ...
	RWGltf_GltfPrimitiveMode_TriangleStrip: int = ...
	RWGltf_GltfPrimitiveMode_TriangleFan: int = ...
RWGltf_GltfPrimitiveMode_UNKNOWN = RWGltf_GltfPrimitiveMode.RWGltf_GltfPrimitiveMode_UNKNOWN
RWGltf_GltfPrimitiveMode_Points = RWGltf_GltfPrimitiveMode.RWGltf_GltfPrimitiveMode_Points
RWGltf_GltfPrimitiveMode_Lines = RWGltf_GltfPrimitiveMode.RWGltf_GltfPrimitiveMode_Lines
RWGltf_GltfPrimitiveMode_LineLoop = RWGltf_GltfPrimitiveMode.RWGltf_GltfPrimitiveMode_LineLoop
RWGltf_GltfPrimitiveMode_LineStrip = RWGltf_GltfPrimitiveMode.RWGltf_GltfPrimitiveMode_LineStrip
RWGltf_GltfPrimitiveMode_Triangles = RWGltf_GltfPrimitiveMode.RWGltf_GltfPrimitiveMode_Triangles
RWGltf_GltfPrimitiveMode_TriangleStrip = RWGltf_GltfPrimitiveMode.RWGltf_GltfPrimitiveMode_TriangleStrip
RWGltf_GltfPrimitiveMode_TriangleFan = RWGltf_GltfPrimitiveMode.RWGltf_GltfPrimitiveMode_TriangleFan

class RWGltf_GltfBufferViewTarget(IntEnum):
	RWGltf_GltfBufferViewTarget_UNKNOWN: int = ...
	RWGltf_GltfBufferViewTarget_ARRAY_BUFFER: int = ...
	RWGltf_GltfBufferViewTarget_ELEMENT_ARRAY_BUFFER: int = ...
RWGltf_GltfBufferViewTarget_UNKNOWN = RWGltf_GltfBufferViewTarget.RWGltf_GltfBufferViewTarget_UNKNOWN
RWGltf_GltfBufferViewTarget_ARRAY_BUFFER = RWGltf_GltfBufferViewTarget.RWGltf_GltfBufferViewTarget_ARRAY_BUFFER
RWGltf_GltfBufferViewTarget_ELEMENT_ARRAY_BUFFER = RWGltf_GltfBufferViewTarget.RWGltf_GltfBufferViewTarget_ELEMENT_ARRAY_BUFFER

class RWGltf_GltfArrayType(IntEnum):
	RWGltf_GltfArrayType_UNKNOWN: int = ...
	RWGltf_GltfArrayType_Indices: int = ...
	RWGltf_GltfArrayType_Position: int = ...
	RWGltf_GltfArrayType_Normal: int = ...
	RWGltf_GltfArrayType_Color: int = ...
	RWGltf_GltfArrayType_TCoord0: int = ...
	RWGltf_GltfArrayType_TCoord1: int = ...
	RWGltf_GltfArrayType_Joint: int = ...
	RWGltf_GltfArrayType_Weight: int = ...
RWGltf_GltfArrayType_UNKNOWN = RWGltf_GltfArrayType.RWGltf_GltfArrayType_UNKNOWN
RWGltf_GltfArrayType_Indices = RWGltf_GltfArrayType.RWGltf_GltfArrayType_Indices
RWGltf_GltfArrayType_Position = RWGltf_GltfArrayType.RWGltf_GltfArrayType_Position
RWGltf_GltfArrayType_Normal = RWGltf_GltfArrayType.RWGltf_GltfArrayType_Normal
RWGltf_GltfArrayType_Color = RWGltf_GltfArrayType.RWGltf_GltfArrayType_Color
RWGltf_GltfArrayType_TCoord0 = RWGltf_GltfArrayType.RWGltf_GltfArrayType_TCoord0
RWGltf_GltfArrayType_TCoord1 = RWGltf_GltfArrayType.RWGltf_GltfArrayType_TCoord1
RWGltf_GltfArrayType_Joint = RWGltf_GltfArrayType.RWGltf_GltfArrayType_Joint
RWGltf_GltfArrayType_Weight = RWGltf_GltfArrayType.RWGltf_GltfArrayType_Weight

class RWGltf_GltfRootElement(IntEnum):
	RWGltf_GltfRootElement_Asset: int = ...
	RWGltf_GltfRootElement_Scenes: int = ...
	RWGltf_GltfRootElement_Scene: int = ...
	RWGltf_GltfRootElement_Nodes: int = ...
	RWGltf_GltfRootElement_Meshes: int = ...
	RWGltf_GltfRootElement_Accessors: int = ...
	RWGltf_GltfRootElement_BufferViews: int = ...
	RWGltf_GltfRootElement_Buffers: int = ...
	RWGltf_GltfRootElement_NB_MANDATORY: int = ...
	RWGltf_GltfRootElement_Animations: int = ...
	RWGltf_GltfRootElement_Materials: int = ...
	RWGltf_GltfRootElement_Programs: int = ...
	RWGltf_GltfRootElement_Samplers: int = ...
	RWGltf_GltfRootElement_Shaders: int = ...
	RWGltf_GltfRootElement_Skins: int = ...
	RWGltf_GltfRootElement_Techniques: int = ...
	RWGltf_GltfRootElement_Textures: int = ...
	RWGltf_GltfRootElement_Images: int = ...
	RWGltf_GltfRootElement_ExtensionsUsed: int = ...
	RWGltf_GltfRootElement_ExtensionsRequired: int = ...
	RWGltf_GltfRootElement_NB: int = ...
RWGltf_GltfRootElement_Asset = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Asset
RWGltf_GltfRootElement_Scenes = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Scenes
RWGltf_GltfRootElement_Scene = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Scene
RWGltf_GltfRootElement_Nodes = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Nodes
RWGltf_GltfRootElement_Meshes = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Meshes
RWGltf_GltfRootElement_Accessors = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Accessors
RWGltf_GltfRootElement_BufferViews = RWGltf_GltfRootElement.RWGltf_GltfRootElement_BufferViews
RWGltf_GltfRootElement_Buffers = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Buffers
RWGltf_GltfRootElement_NB_MANDATORY = RWGltf_GltfRootElement.RWGltf_GltfRootElement_NB_MANDATORY
RWGltf_GltfRootElement_Animations = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Animations
RWGltf_GltfRootElement_Materials = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Materials
RWGltf_GltfRootElement_Programs = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Programs
RWGltf_GltfRootElement_Samplers = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Samplers
RWGltf_GltfRootElement_Shaders = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Shaders
RWGltf_GltfRootElement_Skins = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Skins
RWGltf_GltfRootElement_Techniques = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Techniques
RWGltf_GltfRootElement_Textures = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Textures
RWGltf_GltfRootElement_Images = RWGltf_GltfRootElement.RWGltf_GltfRootElement_Images
RWGltf_GltfRootElement_ExtensionsUsed = RWGltf_GltfRootElement.RWGltf_GltfRootElement_ExtensionsUsed
RWGltf_GltfRootElement_ExtensionsRequired = RWGltf_GltfRootElement.RWGltf_GltfRootElement_ExtensionsRequired
RWGltf_GltfRootElement_NB = RWGltf_GltfRootElement.RWGltf_GltfRootElement_NB

class RWGltf_GltfAccessorCompType(IntEnum):
	RWGltf_GltfAccessorCompType_UNKNOWN: int = ...
	RWGltf_GltfAccessorCompType_Int8: int = ...
	RWGltf_GltfAccessorCompType_UInt8: int = ...
	RWGltf_GltfAccessorCompType_Int16: int = ...
	RWGltf_GltfAccessorCompType_UInt16: int = ...
	RWGltf_GltfAccessorCompType_UInt32: int = ...
	RWGltf_GltfAccessorCompType_Float32: int = ...
RWGltf_GltfAccessorCompType_UNKNOWN = RWGltf_GltfAccessorCompType.RWGltf_GltfAccessorCompType_UNKNOWN
RWGltf_GltfAccessorCompType_Int8 = RWGltf_GltfAccessorCompType.RWGltf_GltfAccessorCompType_Int8
RWGltf_GltfAccessorCompType_UInt8 = RWGltf_GltfAccessorCompType.RWGltf_GltfAccessorCompType_UInt8
RWGltf_GltfAccessorCompType_Int16 = RWGltf_GltfAccessorCompType.RWGltf_GltfAccessorCompType_Int16
RWGltf_GltfAccessorCompType_UInt16 = RWGltf_GltfAccessorCompType.RWGltf_GltfAccessorCompType_UInt16
RWGltf_GltfAccessorCompType_UInt32 = RWGltf_GltfAccessorCompType.RWGltf_GltfAccessorCompType_UInt32
RWGltf_GltfAccessorCompType_Float32 = RWGltf_GltfAccessorCompType.RWGltf_GltfAccessorCompType_Float32

class RWGltf_GltfAlphaMode(IntEnum):
	RWGltf_GltfAlphaMode_Opaque: int = ...
	RWGltf_GltfAlphaMode_Mask: int = ...
	RWGltf_GltfAlphaMode_Blend: int = ...
RWGltf_GltfAlphaMode_Opaque = RWGltf_GltfAlphaMode.RWGltf_GltfAlphaMode_Opaque
RWGltf_GltfAlphaMode_Mask = RWGltf_GltfAlphaMode.RWGltf_GltfAlphaMode_Mask
RWGltf_GltfAlphaMode_Blend = RWGltf_GltfAlphaMode.RWGltf_GltfAlphaMode_Blend

class RWGltf_GltfAccessorLayout(IntEnum):
	RWGltf_GltfAccessorLayout_UNKNOWN: int = ...
	RWGltf_GltfAccessorLayout_Scalar: int = ...
	RWGltf_GltfAccessorLayout_Vec2: int = ...
	RWGltf_GltfAccessorLayout_Vec3: int = ...
	RWGltf_GltfAccessorLayout_Vec4: int = ...
	RWGltf_GltfAccessorLayout_Mat2: int = ...
	RWGltf_GltfAccessorLayout_Mat3: int = ...
	RWGltf_GltfAccessorLayout_Mat4: int = ...
RWGltf_GltfAccessorLayout_UNKNOWN = RWGltf_GltfAccessorLayout.RWGltf_GltfAccessorLayout_UNKNOWN
RWGltf_GltfAccessorLayout_Scalar = RWGltf_GltfAccessorLayout.RWGltf_GltfAccessorLayout_Scalar
RWGltf_GltfAccessorLayout_Vec2 = RWGltf_GltfAccessorLayout.RWGltf_GltfAccessorLayout_Vec2
RWGltf_GltfAccessorLayout_Vec3 = RWGltf_GltfAccessorLayout.RWGltf_GltfAccessorLayout_Vec3
RWGltf_GltfAccessorLayout_Vec4 = RWGltf_GltfAccessorLayout.RWGltf_GltfAccessorLayout_Vec4
RWGltf_GltfAccessorLayout_Mat2 = RWGltf_GltfAccessorLayout.RWGltf_GltfAccessorLayout_Mat2
RWGltf_GltfAccessorLayout_Mat3 = RWGltf_GltfAccessorLayout.RWGltf_GltfAccessorLayout_Mat3
RWGltf_GltfAccessorLayout_Mat4 = RWGltf_GltfAccessorLayout.RWGltf_GltfAccessorLayout_Mat4

class RWGltf_WriterTrsfFormat(IntEnum):
	RWGltf_WriterTrsfFormat_Compact: int = ...
	RWGltf_WriterTrsfFormat_Mat4: int = ...
	RWGltf_WriterTrsfFormat_TRS: int = ...
RWGltf_WriterTrsfFormat_Compact = RWGltf_WriterTrsfFormat.RWGltf_WriterTrsfFormat_Compact
RWGltf_WriterTrsfFormat_Mat4 = RWGltf_WriterTrsfFormat.RWGltf_WriterTrsfFormat_Mat4
RWGltf_WriterTrsfFormat_TRS = RWGltf_WriterTrsfFormat.RWGltf_WriterTrsfFormat_TRS

class RWGltf_CafReader(RWMesh_CafReader):
	def __init__(self) -> None: ...
	def ToParallel(self) -> False: ...
	def ToSkipEmptyNodes(self) -> False: ...
	def ToUseMeshNameAsFallback(self) -> False: ...

class RWGltf_CafWriter(Standard_Transient):
	def __init__(self, theFile: TCollection_AsciiString, theIsBinary: bool) -> None: ...
	def ChangeCoordinateSystemConverter(self) -> False: ...
	def CoordinateSystemConverter(self) -> False: ...
	def DefaultStyle(self) -> XCAFPrs_Style: ...
	def IsBinary(self) -> False: ...
	def IsForcedUVExport(self) -> False: ...
	@overload
	def Perform(self, theDocument: TDocStd_Document, theRootLabels: TDF_LabelSequence, theLabelFilter: TColStd_MapOfAsciiString, theFileInfo: TColStd_IndexedDataMapOfStringString, theProgress: Message_ProgressRange) -> False: ...
	@overload
	def Perform(self, theDocument: TDocStd_Document, theFileInfo: TColStd_IndexedDataMapOfStringString, theProgress: Message_ProgressRange) -> False: ...
	def SetDefaultStyle(self, theStyle: XCAFPrs_Style) -> None: ...
	def SetTransformationFormat(self, theFormat: RWGltf_WriterTrsfFormat) -> None: ...
	def TransformationFormat(self) -> RWGltf_WriterTrsfFormat: ...

class RWGltf_GltfAccessor:
	def __init__(self) -> None: ...

class RWGltf_GltfBufferView:
	def __init__(self) -> None: ...

class RWGltf_GltfFace:
	pass

class RWGltf_GltfLatePrimitiveArray(Poly_Triangulation):
	def __init__(self, theId: TCollection_AsciiString, theName: TCollection_AsciiString) -> None: ...
	def AddPrimArrayData(self, theType: RWGltf_GltfArrayType) -> RWGltf_GltfPrimArrayData: ...
	def BaseColor(self) -> Quantity_ColorRGBA: ...
	def BoundingBox(self) -> Bnd_Box: ...
	def Data(self) -> False: ...
	def HasStyle(self) -> False: ...
	def Id(self) -> TCollection_AsciiString: ...
	def MaterialCommon(self) -> RWGltf_MaterialCommon: ...
	def MaterialPbr(self) -> RWGltf_MaterialMetallicRoughness: ...
	def Name(self) -> TCollection_AsciiString: ...
	def PrimitiveMode(self) -> RWGltf_GltfPrimitiveMode: ...
	def SetBoundingBox(self, theBox: Bnd_Box) -> None: ...
	def SetMaterialCommon(self, theMat: RWGltf_MaterialCommon) -> None: ...
	def SetMaterialPbr(self, theMat: RWGltf_MaterialMetallicRoughness) -> None: ...
	def SetName(self, theName: TCollection_AsciiString) -> None: ...
	def SetPrimitiveMode(self, theMode: RWGltf_GltfPrimitiveMode) -> None: ...

class RWGltf_GltfMaterialMap(RWMesh_MaterialMap):
	def __init__(self, theFile: TCollection_AsciiString, theDefSamplerId: int) -> None: ...
	def AddImages(self, theWriter: RWGltf_GltfOStreamWriter, theStyle: XCAFPrs_Style) -> bool: ...
	def AddMaterial(self, theWriter: RWGltf_GltfOStreamWriter, theStyle: XCAFPrs_Style) -> bool: ...
	def AddTextures(self, theWriter: RWGltf_GltfOStreamWriter, theStyle: XCAFPrs_Style) -> bool: ...
	def NbImages(self) -> int: ...
	def NbTextures(self) -> int: ...
	@staticmethod
	def baseColorTexture(theMat: XCAFDoc_VisMaterial) -> Image_Texture: ...

class RWGltf_GltfOStreamWriter():
	pass

class RWGltf_GltfPrimArrayData:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, theType: RWGltf_GltfArrayType) -> None: ...

class RWGltf_GltfSceneNodeMap():
	def __init__(self) -> None: ...
	def FindIndex(self, theNodeId: TCollection_AsciiString) -> int: ...

class RWGltf_MaterialCommon(Standard_Transient):
	def __init__(self) -> None: ...

class RWGltf_MaterialMetallicRoughness(Standard_Transient):
	def __init__(self) -> None: ...

class RWGltf_PrimitiveArrayReader(Standard_Transient):
	def CoordinateSystemConverter(self) -> False: ...
	def ErrorPrefix(self) -> TCollection_AsciiString: ...
	def Load(self, theMesh: RWGltf_GltfLatePrimitiveArray) -> Poly_Triangulation: ...
	def SetErrorPrefix(self, theErrPrefix: TCollection_AsciiString) -> None: ...

class RWGltf_TriangulationReader(RWGltf_PrimitiveArrayReader):
	def __init__(self) -> None: ...

#classnotwrapped
class RWGltf_GltfSharedIStream: ...

# harray1 classes
# harray2 classes
# hsequence classes

RWGltf_GltfMaterialMap_baseColorTexture = RWGltf_GltfMaterialMap.baseColorTexture



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.TopoDS import *
from OCC.Core.TColgp import *
from OCC.Core.TopTools import *
from OCC.Core.TopAbs import *
from OCC.Core.TColGeom import *
from OCC.Core.gp import *
from OCC.Core.Geom import *


class LocOpe_SequenceOfCirc:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> gp_Circ: ...
    def Last(self) -> gp_Circ: ...
    def Length(self) -> int: ...
    def Append(self, theItem: gp_Circ) -> gp_Circ: ...
    def Prepend(self, theItem: gp_Circ) -> gp_Circ: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> gp_Circ: ...
    def SetValue(self, theIndex: int, theValue: gp_Circ) -> None: ...

class LocOpe_SequenceOfLin:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> gp_Lin: ...
    def Last(self) -> gp_Lin: ...
    def Length(self) -> int: ...
    def Append(self, theItem: gp_Lin) -> gp_Lin: ...
    def Prepend(self, theItem: gp_Lin) -> gp_Lin: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> gp_Lin: ...
    def SetValue(self, theIndex: int, theValue: gp_Lin) -> None: ...

class LocOpe_SequenceOfPntFace:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> LocOpe_PntFace: ...
    def Last(self) -> LocOpe_PntFace: ...
    def Length(self) -> int: ...
    def Append(self, theItem: LocOpe_PntFace) -> LocOpe_PntFace: ...
    def Prepend(self, theItem: LocOpe_PntFace) -> LocOpe_PntFace: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> LocOpe_PntFace: ...
    def SetValue(self, theIndex: int, theValue: LocOpe_PntFace) -> None: ...

class LocOpe_Operation(IntEnum):
	LocOpe_FUSE: int = ...
	LocOpe_CUT: int = ...
	LocOpe_INVALID: int = ...
LocOpe_FUSE = LocOpe_Operation.LocOpe_FUSE
LocOpe_CUT = LocOpe_Operation.LocOpe_CUT
LocOpe_INVALID = LocOpe_Operation.LocOpe_INVALID

class locope:
	@overload
	@staticmethod
	def Closed(W: TopoDS_Wire, OnF: TopoDS_Face) -> bool: ...
	@overload
	@staticmethod
	def Closed(E: TopoDS_Edge, OnF: TopoDS_Face) -> bool: ...
	@staticmethod
	def SampleEdges(S: TopoDS_Shape, Pt: TColgp_SequenceOfPnt) -> None: ...
	@staticmethod
	def TgtFaces(E: TopoDS_Edge, F1: TopoDS_Face, F2: TopoDS_Face) -> bool: ...

class LocOpe_BuildShape:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, L: TopTools_ListOfShape) -> None: ...
	def Perform(self, L: TopTools_ListOfShape) -> None: ...
	def Shape(self) -> TopoDS_Shape: ...

class LocOpe_BuildWires:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, Ledges: TopTools_ListOfShape, PW: LocOpe_WiresOnShape) -> None: ...
	def IsDone(self) -> bool: ...
	def Perform(self, Ledges: TopTools_ListOfShape, PW: LocOpe_WiresOnShape) -> None: ...
	def Result(self) -> TopTools_ListOfShape: ...

class LocOpe_CSIntersector:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, S: TopoDS_Shape) -> None: ...
	def Destroy(self) -> None: ...
	def Init(self, S: TopoDS_Shape) -> None: ...
	def IsDone(self) -> bool: ...
	@overload
	def LocalizeAfter(self, I: int, From: float, Tol: float, Or: TopAbs_Orientation) -> Tuple[bool, int, int]: ...
	@overload
	def LocalizeAfter(self, I: int, FromInd: int, Tol: float, Or: TopAbs_Orientation) -> Tuple[bool, int, int]: ...
	@overload
	def LocalizeBefore(self, I: int, From: float, Tol: float, Or: TopAbs_Orientation) -> Tuple[bool, int, int]: ...
	@overload
	def LocalizeBefore(self, I: int, FromInd: int, Tol: float, Or: TopAbs_Orientation) -> Tuple[bool, int, int]: ...
	def NbPoints(self, I: int) -> int: ...
	@overload
	def Perform(self, Slin: LocOpe_SequenceOfLin) -> None: ...
	@overload
	def Perform(self, Scir: LocOpe_SequenceOfCirc) -> None: ...
	@overload
	def Perform(self, Scur: TColGeom_SequenceOfCurve) -> None: ...
	def Point(self, I: int, Index: int) -> LocOpe_PntFace: ...

class LocOpe_CurveShapeIntersector:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, Axis: gp_Ax1, S: TopoDS_Shape) -> None: ...
	@overload
	def __init__(self, C: gp_Circ, S: TopoDS_Shape) -> None: ...
	@overload
	def Init(self, Axis: gp_Ax1, S: TopoDS_Shape) -> None: ...
	@overload
	def Init(self, C: gp_Circ, S: TopoDS_Shape) -> None: ...
	def IsDone(self) -> bool: ...
	@overload
	def LocalizeAfter(self, From: float, Or: TopAbs_Orientation) -> Tuple[bool, int, int]: ...
	@overload
	def LocalizeAfter(self, FromInd: int, Or: TopAbs_Orientation) -> Tuple[bool, int, int]: ...
	@overload
	def LocalizeBefore(self, From: float, Or: TopAbs_Orientation) -> Tuple[bool, int, int]: ...
	@overload
	def LocalizeBefore(self, FromInd: int, Or: TopAbs_Orientation) -> Tuple[bool, int, int]: ...
	def NbPoints(self) -> int: ...
	def Point(self, Index: int) -> LocOpe_PntFace: ...

class LocOpe_DPrism:
	@overload
	def __init__(self, Spine: TopoDS_Face, Height1: float, Height2: float, Angle: float) -> None: ...
	@overload
	def __init__(self, Spine: TopoDS_Face, Height: float, Angle: float) -> None: ...
	def BarycCurve(self) -> Geom_Curve: ...
	def Curves(self, SCurves: TColGeom_SequenceOfCurve) -> None: ...
	def FirstShape(self) -> TopoDS_Shape: ...
	def IsDone(self) -> bool: ...
	def LastShape(self) -> TopoDS_Shape: ...
	def Profile(self) -> TopoDS_Shape: ...
	def Shape(self) -> TopoDS_Shape: ...
	def Shapes(self, S: TopoDS_Shape) -> TopTools_ListOfShape: ...
	def Spine(self) -> TopoDS_Shape: ...

class LocOpe_FindEdges:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, FFrom: TopoDS_Shape, FTo: TopoDS_Shape) -> None: ...
	def EdgeFrom(self) -> TopoDS_Edge: ...
	def EdgeTo(self) -> TopoDS_Edge: ...
	def InitIterator(self) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def Set(self, FFrom: TopoDS_Shape, FTo: TopoDS_Shape) -> None: ...

class LocOpe_FindEdgesInFace:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, S: TopoDS_Shape, F: TopoDS_Face) -> None: ...
	def Edge(self) -> TopoDS_Edge: ...
	def Init(self) -> None: ...
	def More(self) -> bool: ...
	def Next(self) -> None: ...
	def Set(self, S: TopoDS_Shape, F: TopoDS_Face) -> None: ...

class LocOpe_GeneratedShape(Standard_Transient):
	@overload
	def Generated(self, V: TopoDS_Vertex) -> TopoDS_Edge: ...
	@overload
	def Generated(self, E: TopoDS_Edge) -> TopoDS_Face: ...
	def GeneratingEdges(self) -> TopTools_ListOfShape: ...
	def OrientedFaces(self) -> TopTools_ListOfShape: ...

class LocOpe_Generator:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, S: TopoDS_Shape) -> None: ...
	def DescendantFace(self, F: TopoDS_Face) -> TopTools_ListOfShape: ...
	def Init(self, S: TopoDS_Shape) -> None: ...
	def IsDone(self) -> bool: ...
	def Perform(self, G: LocOpe_GeneratedShape) -> None: ...
	def ResultingShape(self) -> TopoDS_Shape: ...
	def Shape(self) -> TopoDS_Shape: ...

class LocOpe_Gluer:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, Sbase: TopoDS_Shape, Snew: TopoDS_Shape) -> None: ...
	def BasisShape(self) -> TopoDS_Shape: ...
	@overload
	def Bind(self, Fnew: TopoDS_Face, Fbase: TopoDS_Face) -> None: ...
	@overload
	def Bind(self, Enew: TopoDS_Edge, Ebase: TopoDS_Edge) -> None: ...
	def DescendantFaces(self, F: TopoDS_Face) -> TopTools_ListOfShape: ...
	def Edges(self) -> TopTools_ListOfShape: ...
	def GluedShape(self) -> TopoDS_Shape: ...
	def Init(self, Sbase: TopoDS_Shape, Snew: TopoDS_Shape) -> None: ...
	def IsDone(self) -> bool: ...
	def OpeType(self) -> LocOpe_Operation: ...
	def Perform(self) -> None: ...
	def ResultingShape(self) -> TopoDS_Shape: ...
	def TgtEdges(self) -> TopTools_ListOfShape: ...

class LocOpe_LinearForm:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, Base: TopoDS_Shape, V: gp_Vec, Pnt1: gp_Pnt, Pnt2: gp_Pnt) -> None: ...
	@overload
	def __init__(self, Base: TopoDS_Shape, V: gp_Vec, Vectra: gp_Vec, Pnt1: gp_Pnt, Pnt2: gp_Pnt) -> None: ...
	def FirstShape(self) -> TopoDS_Shape: ...
	def LastShape(self) -> TopoDS_Shape: ...
	@overload
	def Perform(self, Base: TopoDS_Shape, V: gp_Vec, Pnt1: gp_Pnt, Pnt2: gp_Pnt) -> None: ...
	@overload
	def Perform(self, Base: TopoDS_Shape, V: gp_Vec, Vectra: gp_Vec, Pnt1: gp_Pnt, Pnt2: gp_Pnt) -> None: ...
	def Shape(self) -> TopoDS_Shape: ...
	def Shapes(self, S: TopoDS_Shape) -> TopTools_ListOfShape: ...

class LocOpe_Pipe:
	def __init__(self, Spine: TopoDS_Wire, Profile: TopoDS_Shape) -> None: ...
	def BarycCurve(self) -> Geom_Curve: ...
	def Curves(self, Spt: TColgp_SequenceOfPnt) -> TColGeom_SequenceOfCurve: ...
	def FirstShape(self) -> TopoDS_Shape: ...
	def LastShape(self) -> TopoDS_Shape: ...
	def Profile(self) -> TopoDS_Shape: ...
	def Shape(self) -> TopoDS_Shape: ...
	def Shapes(self, S: TopoDS_Shape) -> TopTools_ListOfShape: ...
	def Spine(self) -> TopoDS_Shape: ...

class LocOpe_PntFace:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, P: gp_Pnt, F: TopoDS_Face, Or: TopAbs_Orientation, Param: float, UPar: float, VPar: float) -> None: ...
	def ChangeOrientation(self) -> TopAbs_Orientation: ...
	def Face(self) -> TopoDS_Face: ...
	def Orientation(self) -> TopAbs_Orientation: ...
	def Parameter(self) -> float: ...
	def Pnt(self) -> gp_Pnt: ...
	def UParameter(self) -> float: ...
	def VParameter(self) -> float: ...

class LocOpe_Prism:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, Base: TopoDS_Shape, V: gp_Vec) -> None: ...
	@overload
	def __init__(self, Base: TopoDS_Shape, V: gp_Vec, Vectra: gp_Vec) -> None: ...
	def BarycCurve(self) -> Geom_Curve: ...
	def Curves(self, SCurves: TColGeom_SequenceOfCurve) -> None: ...
	def FirstShape(self) -> TopoDS_Shape: ...
	def LastShape(self) -> TopoDS_Shape: ...
	@overload
	def Perform(self, Base: TopoDS_Shape, V: gp_Vec) -> None: ...
	@overload
	def Perform(self, Base: TopoDS_Shape, V: gp_Vec, Vtra: gp_Vec) -> None: ...
	def Shape(self) -> TopoDS_Shape: ...
	def Shapes(self, S: TopoDS_Shape) -> TopTools_ListOfShape: ...

class LocOpe_SplitDrafts:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, S: TopoDS_Shape) -> None: ...
	def Init(self, S: TopoDS_Shape) -> None: ...
	def IsDone(self) -> bool: ...
	def OriginalShape(self) -> TopoDS_Shape: ...
	@overload
	def Perform(self, F: TopoDS_Face, W: TopoDS_Wire, Extractg: gp_Dir, NPlg: gp_Pln, Angleg: float, Extractd: gp_Dir, NPld: gp_Pln, Angled: float, ModifyLeft: Optional[bool] = True, ModifyRight: Optional[bool] = True) -> None: ...
	@overload
	def Perform(self, F: TopoDS_Face, W: TopoDS_Wire, Extract: gp_Dir, NPl: gp_Pln, Angle: float) -> None: ...
	def Shape(self) -> TopoDS_Shape: ...
	def ShapesFromShape(self, S: TopoDS_Shape) -> TopTools_ListOfShape: ...

class LocOpe_SplitShape:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, S: TopoDS_Shape) -> None: ...
	@overload
	def Add(self, V: TopoDS_Vertex, P: float, E: TopoDS_Edge) -> None: ...
	@overload
	def Add(self, W: TopoDS_Wire, F: TopoDS_Face) -> bool: ...
	@overload
	def Add(self, Lwires: TopTools_ListOfShape, F: TopoDS_Face) -> bool: ...
	def CanSplit(self, E: TopoDS_Edge) -> bool: ...
	def DescendantShapes(self, S: TopoDS_Shape) -> TopTools_ListOfShape: ...
	def Init(self, S: TopoDS_Shape) -> None: ...
	def LeftOf(self, W: TopoDS_Wire, F: TopoDS_Face) -> TopTools_ListOfShape: ...
	def Shape(self) -> TopoDS_Shape: ...

class LocOpe_Spliter:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, S: TopoDS_Shape) -> None: ...
	def DescendantShapes(self, S: TopoDS_Shape) -> TopTools_ListOfShape: ...
	def DirectLeft(self) -> TopTools_ListOfShape: ...
	def Init(self, S: TopoDS_Shape) -> None: ...
	def IsDone(self) -> bool: ...
	def Left(self) -> TopTools_ListOfShape: ...
	def Perform(self, PW: LocOpe_WiresOnShape) -> None: ...
	def ResultingShape(self) -> TopoDS_Shape: ...
	def Shape(self) -> TopoDS_Shape: ...

class LocOpe_WiresOnShape(Standard_Transient):
	def __init__(self, S: TopoDS_Shape) -> None: ...
	def Add(self, theEdges: TopTools_SequenceOfShape) -> bool: ...
	@overload
	def Bind(self, W: TopoDS_Wire, F: TopoDS_Face) -> None: ...
	@overload
	def Bind(self, Comp: TopoDS_Compound, F: TopoDS_Face) -> None: ...
	@overload
	def Bind(self, E: TopoDS_Edge, F: TopoDS_Face) -> None: ...
	@overload
	def Bind(self, EfromW: TopoDS_Edge, EonFace: TopoDS_Edge) -> None: ...
	def BindAll(self) -> None: ...
	def Edge(self) -> TopoDS_Edge: ...
	def Init(self, S: TopoDS_Shape) -> None: ...
	def InitEdgeIterator(self) -> None: ...
	def IsDone(self) -> bool: ...
	def IsFaceWithSection(self, aFace: TopoDS_Shape) -> bool: ...
	def MoreEdge(self) -> bool: ...
	def NextEdge(self) -> None: ...
	@overload
	def OnEdge(self, E: TopoDS_Edge) -> bool: ...
	@overload
	def OnEdge(self, V: TopoDS_Vertex, E: TopoDS_Edge) -> Tuple[bool, float]: ...
	@overload
	def OnEdge(self, V: TopoDS_Vertex, EdgeFrom: TopoDS_Edge, E: TopoDS_Edge) -> Tuple[bool, float]: ...
	def OnFace(self) -> TopoDS_Face: ...
	def OnVertex(self, Vwire: TopoDS_Vertex, Vshape: TopoDS_Vertex) -> bool: ...
	def SetCheckInterior(self, ToCheckInterior: bool) -> None: ...

class LocOpe_GluedShape(LocOpe_GeneratedShape):
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, S: TopoDS_Shape) -> None: ...
	@overload
	def Generated(self, V: TopoDS_Vertex) -> TopoDS_Edge: ...
	@overload
	def Generated(self, E: TopoDS_Edge) -> TopoDS_Face: ...
	def GeneratingEdges(self) -> TopTools_ListOfShape: ...
	def GlueOnFace(self, F: TopoDS_Face) -> None: ...
	def Init(self, S: TopoDS_Shape) -> None: ...
	def OrientedFaces(self) -> TopTools_ListOfShape: ...

#classnotwrapped
class LocOpe_Revol: ...

#classnotwrapped
class LocOpe_RevolutionForm: ...

# harray1 classes
# harray2 classes
# hsequence classes

locope_Closed = locope.Closed
locope_Closed = locope.Closed
locope_SampleEdges = locope.SampleEdges
locope_TgtFaces = locope.TgtFaces



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.TCollection import *
from OCC.Core.gp import *


class XCAFView_ProjectionType(IntEnum):
	XCAFView_ProjectionType_NoCamera: int = ...
	XCAFView_ProjectionType_Parallel: int = ...
	XCAFView_ProjectionType_Central: int = ...
XCAFView_ProjectionType_NoCamera = XCAFView_ProjectionType.XCAFView_ProjectionType_NoCamera
XCAFView_ProjectionType_Parallel = XCAFView_ProjectionType.XCAFView_ProjectionType_Parallel
XCAFView_ProjectionType_Central = XCAFView_ProjectionType.XCAFView_ProjectionType_Central

class XCAFView_Object(Standard_Transient):
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, theObj: XCAFView_Object) -> None: ...
	def BackPlaneDistance(self) -> float: ...
	def ClippingExpression(self) -> TCollection_HAsciiString: ...
	def CreateGDTPoints(self, theLenght: int) -> None: ...
	def FrontPlaneDistance(self) -> float: ...
	def GDTPoint(self, theIndex: int) -> gp_Pnt: ...
	def HasBackPlaneClipping(self) -> bool: ...
	def HasFrontPlaneClipping(self) -> bool: ...
	def HasGDTPoints(self) -> bool: ...
	def HasViewVolumeSidesClipping(self) -> bool: ...
	def Name(self) -> TCollection_HAsciiString: ...
	def NbGDTPoints(self) -> int: ...
	def ProjectionPoint(self) -> gp_Pnt: ...
	def SetBackPlaneDistance(self, theDistance: float) -> None: ...
	def SetClippingExpression(self, theExpression: TCollection_HAsciiString) -> None: ...
	def SetFrontPlaneDistance(self, theDistance: float) -> None: ...
	def SetGDTPoint(self, theIndex: int, thePoint: gp_Pnt) -> None: ...
	def SetName(self, theName: TCollection_HAsciiString) -> None: ...
	def SetProjectionPoint(self, thePoint: gp_Pnt) -> None: ...
	def SetType(self, theType: XCAFView_ProjectionType) -> None: ...
	def SetUpDirection(self, theDirection: gp_Dir) -> None: ...
	def SetViewDirection(self, theDirection: gp_Dir) -> None: ...
	def SetViewVolumeSidesClipping(self, theViewVolumeSidesClipping: bool) -> None: ...
	def SetWindowHorizontalSize(self, theSize: float) -> None: ...
	def SetWindowVerticalSize(self, theSize: float) -> None: ...
	def SetZoomFactor(self, theZoomFactor: float) -> None: ...
	def Type(self) -> XCAFView_ProjectionType: ...
	def UnsetBackPlaneClipping(self) -> None: ...
	def UnsetFrontPlaneClipping(self) -> None: ...
	def UpDirection(self) -> gp_Dir: ...
	def ViewDirection(self) -> gp_Dir: ...
	def WindowHorizontalSize(self) -> float: ...
	def WindowVerticalSize(self) -> float: ...
	def ZoomFactor(self) -> float: ...

# harray1 classes
# harray2 classes
# hsequence classes




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
XmlMXCAFDoc module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_xmlmxcafdoc.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _XmlMXCAFDoc
else:
    import _XmlMXCAFDoc

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _XmlMXCAFDoc.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _XmlMXCAFDoc.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _XmlMXCAFDoc.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _XmlMXCAFDoc.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _XmlMXCAFDoc.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _XmlMXCAFDoc.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _XmlMXCAFDoc.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _XmlMXCAFDoc.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _XmlMXCAFDoc.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _XmlMXCAFDoc.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _XmlMXCAFDoc.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _XmlMXCAFDoc.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _XmlMXCAFDoc.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _XmlMXCAFDoc.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _XmlMXCAFDoc.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _XmlMXCAFDoc.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _XmlMXCAFDoc.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _XmlMXCAFDoc:
_XmlMXCAFDoc.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _XmlMXCAFDoc.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.XmlMDF
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.TDF
import OCC.Core.XmlObjMgt
import OCC.Core.LDOM
import OCC.Core.gp
import OCC.Core.Storage
import OCC.Core.TopTools
import OCC.Core.TopoDS
import OCC.Core.TopAbs
import OCC.Core.TopLoc

from enum import IntEnum
from OCC.Core.Exception import *




def Handle_XmlMXCAFDoc_AssemblyItemRefDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_AssemblyItemRefDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_AssemblyItemRefDriver_Create()

def Handle_XmlMXCAFDoc_AssemblyItemRefDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_AssemblyItemRefDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_AssemblyItemRefDriver_DownCast(t)

def Handle_XmlMXCAFDoc_AssemblyItemRefDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_AssemblyItemRefDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_AssemblyItemRefDriver_IsNull(t)

def Handle_XmlMXCAFDoc_CentroidDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_CentroidDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_CentroidDriver_Create()

def Handle_XmlMXCAFDoc_CentroidDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_CentroidDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_CentroidDriver_DownCast(t)

def Handle_XmlMXCAFDoc_CentroidDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_CentroidDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_CentroidDriver_IsNull(t)

def Handle_XmlMXCAFDoc_ColorDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_ColorDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_ColorDriver_Create()

def Handle_XmlMXCAFDoc_ColorDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_ColorDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_ColorDriver_DownCast(t)

def Handle_XmlMXCAFDoc_ColorDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_ColorDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_ColorDriver_IsNull(t)

def Handle_XmlMXCAFDoc_DatumDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_DatumDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_DatumDriver_Create()

def Handle_XmlMXCAFDoc_DatumDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_DatumDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_DatumDriver_DownCast(t)

def Handle_XmlMXCAFDoc_DatumDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_DatumDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_DatumDriver_IsNull(t)

def Handle_XmlMXCAFDoc_DimTolDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_DimTolDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_DimTolDriver_Create()

def Handle_XmlMXCAFDoc_DimTolDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_DimTolDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_DimTolDriver_DownCast(t)

def Handle_XmlMXCAFDoc_DimTolDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_DimTolDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_DimTolDriver_IsNull(t)

def Handle_XmlMXCAFDoc_GraphNodeDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_GraphNodeDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_GraphNodeDriver_Create()

def Handle_XmlMXCAFDoc_GraphNodeDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_GraphNodeDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_GraphNodeDriver_DownCast(t)

def Handle_XmlMXCAFDoc_GraphNodeDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_GraphNodeDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_GraphNodeDriver_IsNull(t)

def Handle_XmlMXCAFDoc_LocationDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_LocationDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_LocationDriver_Create()

def Handle_XmlMXCAFDoc_LocationDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_LocationDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_LocationDriver_DownCast(t)

def Handle_XmlMXCAFDoc_LocationDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_LocationDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_LocationDriver_IsNull(t)

def Handle_XmlMXCAFDoc_MaterialDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_MaterialDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_MaterialDriver_Create()

def Handle_XmlMXCAFDoc_MaterialDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_MaterialDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_MaterialDriver_DownCast(t)

def Handle_XmlMXCAFDoc_MaterialDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_MaterialDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_MaterialDriver_IsNull(t)

def Handle_XmlMXCAFDoc_NoteDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_NoteDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteDriver_Create()

def Handle_XmlMXCAFDoc_NoteDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_NoteDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteDriver_DownCast(t)

def Handle_XmlMXCAFDoc_NoteDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_NoteDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteDriver_IsNull(t)

def Handle_XmlMXCAFDoc_VisMaterialDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_VisMaterialDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_VisMaterialDriver_Create()

def Handle_XmlMXCAFDoc_VisMaterialDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_VisMaterialDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_VisMaterialDriver_DownCast(t)

def Handle_XmlMXCAFDoc_VisMaterialDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_VisMaterialDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_VisMaterialDriver_IsNull(t)

def Handle_XmlMXCAFDoc_VisMaterialToolDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_VisMaterialToolDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_VisMaterialToolDriver_Create()

def Handle_XmlMXCAFDoc_VisMaterialToolDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_VisMaterialToolDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_VisMaterialToolDriver_DownCast(t)

def Handle_XmlMXCAFDoc_VisMaterialToolDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_VisMaterialToolDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_VisMaterialToolDriver_IsNull(t)

def Handle_XmlMXCAFDoc_NoteBinDataDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_NoteBinDataDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteBinDataDriver_Create()

def Handle_XmlMXCAFDoc_NoteBinDataDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_NoteBinDataDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteBinDataDriver_DownCast(t)

def Handle_XmlMXCAFDoc_NoteBinDataDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_NoteBinDataDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteBinDataDriver_IsNull(t)

def Handle_XmlMXCAFDoc_NoteCommentDriver_Create() -> "opencascade::handle< XmlMXCAFDoc_NoteCommentDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteCommentDriver_Create()

def Handle_XmlMXCAFDoc_NoteCommentDriver_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< XmlMXCAFDoc_NoteCommentDriver >":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteCommentDriver_DownCast(t)

def Handle_XmlMXCAFDoc_NoteCommentDriver_IsNull(t: "opencascade::handle< XmlMXCAFDoc_NoteCommentDriver > const &") -> "bool":
    return _XmlMXCAFDoc.Handle_XmlMXCAFDoc_NoteCommentDriver_IsNull(t)
class xmlmxcafdoc(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def AddDrivers(*args) -> "void":
        r"""
        Adds the attribute drivers to <adrivertable>.

        Parameters
        ----------
        aDriverTable: XmlMDF_ADriverTable
        anMsgDrv: Message_Messenger

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.xmlmxcafdoc_AddDrivers(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _XmlMXCAFDoc.xmlmxcafdoc_swiginit(self, _XmlMXCAFDoc.new_xmlmxcafdoc())
    __swig_destroy__ = _XmlMXCAFDoc.delete_xmlmxcafdoc

# Register xmlmxcafdoc in _XmlMXCAFDoc:
_XmlMXCAFDoc.xmlmxcafdoc_swigregister(xmlmxcafdoc)

def xmlmxcafdoc_AddDrivers(*args) -> "void":
    r"""
    Adds the attribute drivers to <adrivertable>.

    Parameters
    ----------
    aDriverTable: XmlMDF_ADriverTable
    anMsgDrv: Message_Messenger

    Returns
    -------
    None

    """
    return _XmlMXCAFDoc.xmlmxcafdoc_AddDrivers(*args)

class XmlMXCAFDoc_AssemblyItemRefDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_AssemblyItemRefDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_AssemblyItemRefDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        theSource: XmlObjMgt_Persistent
        theTarget: TDF_Attribute
        theRelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        theSource: TDF_Attribute
        theTarget: XmlObjMgt_Persistent
        theRelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_AssemblyItemRefDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_AssemblyItemRefDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_AssemblyItemRefDriver

# Register XmlMXCAFDoc_AssemblyItemRefDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_AssemblyItemRefDriver_swigregister(XmlMXCAFDoc_AssemblyItemRefDriver)

class XmlMXCAFDoc_CentroidDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_CentroidDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_CentroidDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_CentroidDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_CentroidDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_CentroidDriver

# Register XmlMXCAFDoc_CentroidDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_CentroidDriver_swigregister(XmlMXCAFDoc_CentroidDriver)

class XmlMXCAFDoc_ColorDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_ColorDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_ColorDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_ColorDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_ColorDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_ColorDriver

# Register XmlMXCAFDoc_ColorDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_ColorDriver_swigregister(XmlMXCAFDoc_ColorDriver)

class XmlMXCAFDoc_DatumDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_DatumDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_DatumDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_DatumDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_DatumDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_DatumDriver

# Register XmlMXCAFDoc_DatumDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_DatumDriver_swigregister(XmlMXCAFDoc_DatumDriver)

class XmlMXCAFDoc_DimTolDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_DimTolDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_DimTolDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_DimTolDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_DimTolDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_DimTolDriver

# Register XmlMXCAFDoc_DimTolDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_DimTolDriver_swigregister(XmlMXCAFDoc_DimTolDriver)

class XmlMXCAFDoc_GraphNodeDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_GraphNodeDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_GraphNodeDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_GraphNodeDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_GraphNodeDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_GraphNodeDriver

# Register XmlMXCAFDoc_GraphNodeDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_GraphNodeDriver_swigregister(XmlMXCAFDoc_GraphNodeDriver)

class XmlMXCAFDoc_LocationDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_LocationDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_LocationDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_LocationDriver_Paste(self, *args)

    def SetSharedLocations(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        theLocations: TopTools_LocationSetPtr

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_LocationDriver_SetSharedLocations(self, *args)

    def Translate(self, *args) -> "Standard_Boolean":
        r"""
        Translate a non storable location to a storable location.

        Parameters
        ----------
        theLoc: TopLoc_Location
        theParent: XmlObjMgt_Element
        theMap: XmlObjMgt_SRelocationTable

        Returns
        -------
        None
        Translate a storable location to a non storable location.

        Parameters
        ----------
        theParent: XmlObjMgt_Element
        theLoc: TopLoc_Location
        theMap: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_LocationDriver_Translate(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_LocationDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_LocationDriver

# Register XmlMXCAFDoc_LocationDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_LocationDriver_swigregister(XmlMXCAFDoc_LocationDriver)

class XmlMXCAFDoc_MaterialDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_MaterialDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_MaterialDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        Source: XmlObjMgt_Persistent
        Target: TDF_Attribute
        RelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        Source: TDF_Attribute
        Target: XmlObjMgt_Persistent
        RelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_MaterialDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_MaterialDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_MaterialDriver

# Register XmlMXCAFDoc_MaterialDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_MaterialDriver_swigregister(XmlMXCAFDoc_MaterialDriver)

class XmlMXCAFDoc_NoteDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        theSource: XmlObjMgt_Persistent
        theTarget: TDF_Attribute
        theRelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        theSource: TDF_Attribute
        theTarget: XmlObjMgt_Persistent
        theRelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_NoteDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_NoteDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_NoteDriver

# Register XmlMXCAFDoc_NoteDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_NoteDriver_swigregister(XmlMXCAFDoc_NoteDriver)

class XmlMXCAFDoc_VisMaterialDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Main constructor.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_VisMaterialDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_VisMaterialDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        Paste attribute from persistence into document.

        Parameters
        ----------
        theSource: XmlObjMgt_Persistent
        theTarget: TDF_Attribute
        theRelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        Paste attribute from document into persistence.

        Parameters
        ----------
        theSource: TDF_Attribute
        theTarget: XmlObjMgt_Persistent
        theRelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_VisMaterialDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_VisMaterialDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_VisMaterialDriver

# Register XmlMXCAFDoc_VisMaterialDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_VisMaterialDriver_swigregister(XmlMXCAFDoc_VisMaterialDriver)

class XmlMXCAFDoc_VisMaterialToolDriver(OCC.Core.XmlMDF.XmlMDF_ADriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Main constructor.

        Parameters
        ----------
        theMsgDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_VisMaterialToolDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_VisMaterialToolDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        Paste attribute from persistence into document.

        Parameters
        ----------
        theSource: XmlObjMgt_Persistent
        theTarget: TDF_Attribute
        theRelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        Paste attribute from document into persistence.

        Parameters
        ----------
        theSource: TDF_Attribute
        theTarget: XmlObjMgt_Persistent
        theRelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_VisMaterialToolDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_VisMaterialToolDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_VisMaterialToolDriver

# Register XmlMXCAFDoc_VisMaterialToolDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_VisMaterialToolDriver_swigregister(XmlMXCAFDoc_VisMaterialToolDriver)

class XmlMXCAFDoc_NoteBinDataDriver(XmlMXCAFDoc_NoteDriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_NoteBinDataDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_NoteBinDataDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        theSource: XmlObjMgt_Persistent
        theTarget: TDF_Attribute
        theRelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        theSource: TDF_Attribute
        theTarget: XmlObjMgt_Persistent
        theRelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_NoteBinDataDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_NoteBinDataDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_NoteBinDataDriver

# Register XmlMXCAFDoc_NoteBinDataDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_NoteBinDataDriver_swigregister(XmlMXCAFDoc_NoteBinDataDriver)

class XmlMXCAFDoc_NoteCommentDriver(XmlMXCAFDoc_NoteDriver):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Parameters
        ----------
        theMessageDriver: Message_Messenger

        Returns
        -------
        None

        """
        _XmlMXCAFDoc.XmlMXCAFDoc_NoteCommentDriver_swiginit(self, _XmlMXCAFDoc.new_XmlMXCAFDoc_NoteCommentDriver(*args))

    def Paste(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        theSource: XmlObjMgt_Persistent
        theTarget: TDF_Attribute
        theRelocTable: XmlObjMgt_RRelocationTable

        Returns
        -------
        bool
        No available documentation.

        Parameters
        ----------
        theSource: TDF_Attribute
        theTarget: XmlObjMgt_Persistent
        theRelocTable: XmlObjMgt_SRelocationTable

        Returns
        -------
        None

        """
        return _XmlMXCAFDoc.XmlMXCAFDoc_NoteCommentDriver_Paste(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_XmlMXCAFDoc_NoteCommentDriver_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _XmlMXCAFDoc.delete_XmlMXCAFDoc_NoteCommentDriver

# Register XmlMXCAFDoc_NoteCommentDriver in _XmlMXCAFDoc:
_XmlMXCAFDoc.XmlMXCAFDoc_NoteCommentDriver_swigregister(XmlMXCAFDoc_NoteCommentDriver)








##Copyright 2009-2014 Jelle Ferina (jelleferinga@gmail.com)
##
##This file is part of pythonOCC.
##
##pythonOCC is free software: you can redistribute it and/or modify
##it under the terms of the GNU Lesser General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##pythonOCC is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Lesser General Public License for more details.
##
##You should have received a copy of the GNU Lesser General Public License
##along with pythonOCC.  If not, see <http://www.gnu.org/licenses/>.

from math import radians

from OCC.Core.BRepBndLib import brepbndlib_Add, brepbndlib_AddOptimal, brepbndlib_AddOBB
from OCC.Core.BRepPrimAPI import BRepPrimAPI_MakeBox, BRepPrimAPI_MakePrism
from OCC.Core.BRepBuilderAPI import (BRepBuilderAPI_MakeEdge,
                                     BRepBuilderAPI_MakeVertex,
                                     BRepBuilderAPI_MakeWire,
                                     BRepBuilderAPI_MakeFace, BRepBuilderAPI_MakeEdge2d,
                                     BRepBuilderAPI_Transform)
from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_GTransform
from OCC.Core.BRepFill import BRepFill_Filling
from OCC.Core.Bnd import Bnd_Box, Bnd_OBB
from OCC.Core.GeomAbs import (GeomAbs_C0, GeomAbs_Plane, GeomAbs_Cylinder, GeomAbs_Cone,
                              GeomAbs_Sphere, GeomAbs_Torus, GeomAbs_BezierSurface, GeomAbs_BSplineSurface,
                              GeomAbs_SurfaceOfRevolution, GeomAbs_SurfaceOfExtrusion, GeomAbs_OffsetSurface,
                              GeomAbs_OtherSurface)
from OCC.Core.BRepAdaptor import BRepAdaptor_Surface, BRepAdaptor_Curve
from OCC.Core.GeomAPI import GeomAPI_PointsToBSpline
from OCC.Core.GProp import GProp_GProps
from OCC.Core.BRepGProp import (brepgprop_LinearProperties,
                                brepgprop_SurfaceProperties,
                                brepgprop_VolumeProperties)
from OCC.Core.TColgp import TColgp_Array1OfPnt
from OCC.Core.TopoDS import TopoDS_Face
from OCC.Core.gp import (gp_Vec, gp_Pnt, gp_Trsf, gp_OX, gp_OY,
                         gp_OZ, gp_XYZ, gp_Ax2, gp_Dir, gp_GTrsf, gp_Mat)

from OCC.Extend.TopologyUtils import is_edge, is_face

#
# assert utils
#
def assert_shape_not_null(shp):
    if shp is None:
        raise AssertionError("Shape is Null.")


def assert_isdone(inst, message):
    if not inst.IsDone():
        raise AssertionError(message)


def point_list_to_TColgp_Array1OfPnt(li):
    pts = TColgp_Array1OfPnt(0, len(li) - 1)
    for n, i in enumerate(li):
        pts.SetValue(n, i)
    return pts

#
# 0D
def make_vertex(*args):
    vert = BRepBuilderAPI_MakeVertex(*args)
    assert_isdone(vert, 'failed to produce edge')
    result = vert.Vertex()
    return result

#
# 1D
#
def make_edge(*args):
    edge = BRepBuilderAPI_MakeEdge(*args)
    assert_isdone(edge, 'failed to produce edge')
    result = edge.Edge()
    return result


def make_edge2d(*args):
    edge = BRepBuilderAPI_MakeEdge2d(*args)
    assert_isdone(edge, 'failed to produce edge')
    result = edge.Edge()
    return result


def make_wire(*args):
    # if we get an iterable, than add all edges to wire builder
    if isinstance(args[0], (list, tuple)):
        wire = BRepBuilderAPI_MakeWire()
        for i in args[0]:
            wire.Add(i)
        wire.Build()
        return wire.Wire()
    wire = BRepBuilderAPI_MakeWire(*args)
    assert_isdone(wire, 'failed to produce wire')
    return wire.Wire()


def points_to_bspline(pnts):
    pts = TColgp_Array1OfPnt(0, len(pnts) - 1)
    for n, i in enumerate(pnts):
        pts.SetValue(n, i)
    crv = GeomAPI_PointsToBSpline(pts)
    return crv.Curve()


def edge_to_bezier(topods_edge):
    """ take an edge and returns:
    * a bool is_bezier
    * the bezier curve
    * degrees
    * poles
    """
    ad = BRepAdaptor_Curve(topods_edge)
    if ad.IsRational():
        return True, ad.Bezier(), ad.Degree()
    else:
        return False, None, None
#
# 2D
#
def make_n_sided(edges, continuity=GeomAbs_C0):
    n_sided = BRepFill_Filling()
    for edg in edges:
        n_sided.Add(edg, continuity)
    n_sided.Build()
    assert_isdone(n_sided, 'failed to produce n_sided')
    face = n_sided.Face()
    return face


def make_face(*args):
    face = BRepBuilderAPI_MakeFace(*args)
    assert_isdone(face, 'failed to produce face')
    result = face.Face()
    return result


def get_aligned_boundingbox(shape, tol=1e-6, optimal_BB=True):
    """ return the bounding box of the TopoDS_Shape `shape`

    Parameters
    ----------

    shape : TopoDS_Shape or a subclass such as TopoDS_Face
        the shape to compute the bounding box from

    tol: float
        tolerance of the computed boundingbox

    use_triangulation : bool, True by default
        This makes the computation more accurate

    Returns
    -------
        if `as_pnt` is True, return a tuple of gp_Pnt instances
         for the lower and another for the upper X,Y,Z values representing the bounding box

        if `as_pnt` is False, return a tuple of lower and then upper X,Y,Z values
         representing the bounding box
    """
    bbox = Bnd_Box()
    bbox.SetGap(tol)

    # note: useTriangulation is True by default, we set it explicitely, but t's not necessary
    if optimal_BB:
        use_triangulation = True
        use_shapetolerance = True
        brepbndlib_AddOptimal(shape, bbox, use_triangulation, use_shapetolerance)
    else:
        brepbndlib_Add(shape, bbox)
    xmin, ymin, zmin, xmax, ymax, zmax = bbox.Get()
    corner1 = gp_Pnt(xmin, ymin, zmin)
    corner2 = gp_Pnt(xmax, ymax, zmax)
    center = midpoint(corner1, corner2)
    dx = xmax - xmin
    dy = ymax - ymin
    dz = zmax - zmin
    box_shp = BRepPrimAPI_MakeBox(corner1, corner2).Shape()
    return center, [dx, dy, dz], box_shp


def get_oriented_boundingbox(shape, optimal_OBB=True):
    """ return the oriented bounding box of the TopoDS_Shape `shape`

    Parameters
    ----------

    shape : TopoDS_Shape or a subclass such as TopoDS_Face
        the shape to compute the bounding box from
    optimal_OBB : bool, True by default. If set to True, compute the
        optimal (i.e. the smallest oriented bounding box). Optimal OBB is
        a bit longer.
    Returns
    -------
        a list with center, x, y and z sizes

        a shape
    """
    obb = Bnd_OBB()
    if optimal_OBB:
        is_triangulationUsed = True
        is_optimal = True
        is_shapeToleranceUsed = False
        brepbndlib_AddOBB(shape, obb, is_triangulationUsed, is_optimal, is_shapeToleranceUsed)
    else:
        brepbndlib_AddOBB(shape, obb)

    # converts the bounding box to a shape
    aBaryCenter = obb.Center()
    aXDir = obb.XDirection()
    aYDir = obb.YDirection()
    aZDir = obb.ZDirection()
    aHalfX = obb.XHSize()
    aHalfY = obb.YHSize()
    aHalfZ = obb.ZHSize()

    ax = gp_XYZ(aXDir.X(), aXDir.Y(), aXDir.Z())
    ay = gp_XYZ(aYDir.X(), aYDir.Y(), aYDir.Z())
    az = gp_XYZ(aZDir.X(), aZDir.Y(), aZDir.Z())
    p = gp_Pnt(aBaryCenter.X(), aBaryCenter.Y(), aBaryCenter.Z())
    anAxes = gp_Ax2(p, gp_Dir(aZDir), gp_Dir(aXDir))
    anAxes.SetLocation(gp_Pnt(p.XYZ() - ax*aHalfX - ay*aHalfY - az*aHalfZ))
    aBox = BRepPrimAPI_MakeBox(anAxes, 2.0*aHalfX, 2.0*aHalfY, 2.0*aHalfZ).Shape()
    return aBaryCenter, [aHalfX, aHalfY, aHalfZ], aBox


def midpoint(pntA, pntB):
    """ computes the point that lies in the middle between pntA and pntB

    Parameters
    ----------

    pntA, pntB : gp_Pnt

    Returns
    -------

    gp_Pnt

    """
    vec1 = gp_Vec(pntA.XYZ())
    vec2 = gp_Vec(pntB.XYZ())
    veccie = (vec1 + vec2) * 0.5
    return gp_Pnt(veccie.XYZ())


def center_boundingbox(shape):
    """ compute the center point of a TopoDS_Shape, based on its bounding box

    Parameters
    ----------

    shape : TopoDS_Shape instance or a subclass like TopoDS_Face

    Returns
    -------

    gp_Pnt

    """
    xmin, ymin, zmin, xmax, ymax, zmax = get_boundingbox(shape, 1e-6)
    return midpoint(gp_Pnt(xmin, ymin, zmin), gp_Pnt(xmax, ymax, zmax))


def translate_shp(shp, vec, copy=False):
    trns = gp_Trsf()
    trns.SetTranslation(vec)
    brep_trns = BRepBuilderAPI_Transform(shp, trns, copy)
    brep_trns.Build()
    return brep_trns.Shape()


def rotate_shape(shape, axis, angle, unite="deg"):
    """ Rotate a shape around an axis, with a given angle.

    @param shape : the shape to rotate
    @point : the origin of the axis
    @vector : the axis direction
    @angle : the value of the rotation

    @return: the rotated shape.
    """
    assert_shape_not_null(shape)
    if unite == "deg":  # convert angle to radians
        angle = radians(angle)
    trns = gp_Trsf()
    trns.SetRotation(axis, angle)
    brep_trns = BRepBuilderAPI_Transform(shape, trns, False)
    brep_trns.Build()
    shp = brep_trns.Shape()
    return shp


def rotate_shp_3_axis(shape, rx, ry, rz, unity="deg"):
    """ Rotate a shape around (O,x), (O,y) and (O,z).

    @param rx_degree : rotation around (O,x)
    @param ry_degree : rotation around (O,y)
    @param rz_degree : rotation around (O,z)

    @return : the rotated shape.
    """
    assert_shape_not_null(shape)
    if unity == "deg":  # convert angle to radians
        rx = radians(rx)
        ry = radians(ry)
        rz = radians(rz)
    alpha = gp_Trsf()
    alpha.SetRotation(gp_OX(), rx)
    beta = gp_Trsf()
    beta.SetRotation(gp_OY(), ry)
    gamma = gp_Trsf()
    gamma.SetRotation(gp_OZ(), rz)
    brep_trns = BRepBuilderAPI_Transform(shape, alpha*beta*gamma, False)
    shp = brep_trns.Shape()
    return shp


def scale_shape(shape, fx, fy, fz):
    """ Scale a shape along the 3 directions
    @param fx : scale factor in the x direction
    @param fy : scale factor in the y direction
    @param fz : scale factor in the z direction

    @return : the scaled shape
    """
    assert_shape_not_null(shape)
    scale_trsf = gp_GTrsf()
    rot = gp_Mat(fx, 0., 0., 0., fy, 0., 0., 0., fz)
    scale_trsf.SetVectorialPart(rot)
    shp = BRepBuilderAPI_GTransform(shape, scale_trsf).Shape()
    return shp


def make_extrusion(face, length, vector=gp_Vec(0., 0., 1.)):
    ''' creates a extrusion from a face, along the vector vector.
    with a distance legnth. Note that the normal vector does not
    necessary be normalized.
    By default, the extrusion is along the z axis.
    '''
    vector.Normalize()
    vector.Scale(length)
    return BRepPrimAPI_MakePrism(face, vector).Shape()

##################################
# Recognize functions
##################################
def recognize_face(topods_face):
    """ returns True if the TopoDS_Face is a planar surface
    """
    if not isinstance(topods_face, TopoDS_Face):
        return "Not a face", None, None
    surf = BRepAdaptor_Surface(topods_face, True)
    surf_type = surf.GetType()
    if  surf_type == GeomAbs_Plane:
        kind = "Plane"
        # look for the properties of the plane
        # first get the related gp_Pln
        gp_pln = surf.Plane()
        location = gp_pln.Location()  # a point of the plane
        normal = gp_pln.Axis().Direction()  # the plane normal
        return kind, location, normal
    elif surf_type == GeomAbs_Cylinder:
        kind = "Cylinder"
        # look for the properties of the cylinder
        # first get the related gp_Cyl
        gp_cyl = surf.Cylinder()
        location = gp_cyl.Location()  # a point of the axis
        axis = gp_cyl.Axis().Direction()  # the cylinder axis
        # then export location and normal to the console output
        return kind, location, axis
    elif surf_type == GeomAbs_Cone:
        kind = "Cone"
        return kind, None, None
    elif surf_type == GeomAbs_Sphere:
        kind = "Sphere"
        return kind, None, None
    elif surf_type == GeomAbs_Torus:
        kind = "Torus"
        return kind, None, None
    elif surf_type == GeomAbs_BezierSurface:
        kind = "Bezier"
        return kind, None, None
    elif surf_type == GeomAbs_BSplineSurface:
        kind = "BSpline"
        return kind, None, None
    elif surf_type == GeomAbs_SurfaceOfRevolution:
        kind = "Revolution"
        return kind, None, None
    elif surf_type == GeomAbs_SurfaceOfExtrusion:
        kind = "Extrusion"
        return kind, None, None
    elif surf_type == GeomAbs_OffsetSurface:
        kind = "Offset"
        return kind, None, None
    elif surf_type == GeomAbs_OtherSurface:
        kind = "Other"
        return kind, None, None
    # nothing found
    return "Unknown", None, None

##############################################################################
# Measure functions
##############################################################################
def measure_shape_volume(shape):
    """ Returns shape volume """
    inertia_props = GProp_GProps()
    brepgprop_VolumeProperties(shape, inertia_props)
    mass = inertia_props.Mass()
    return mass


def measure_shape_mass_center_of_gravity(shape):
    """ Returns the shape center of gravity
    Returns a gp_Pnt if requested (set as_Pnt to True)
    or a list of 3 coordinates, by default."""
    inertia_props = GProp_GProps()
    if is_edge(shape):
        brepgprop_LinearProperties(shape, inertia_props)
        mass_property = "Length"
    elif is_face(shape):
        brepgprop_SurfaceProperties(shape, inertia_props)
        mass_property = "Area"
    else:
        brepgprop_VolumeProperties(shape, inertia_props)
        mass_property = "Volume"
    cog = inertia_props.CentreOfMass()
    mass = inertia_props.Mass()
    return cog, mass, mass_property
